@@ -9,6 +9,12 @@
 
 namespace infomap {
 
+constexpr int FlowModel::undirected;
+constexpr int FlowModel::directed;
+constexpr int FlowModel::undirdir;
+constexpr int FlowModel::outdirdir;
+constexpr int FlowModel::rawdir;
+
 Config::Config(std::string flags, bool isCLI) : isCLI(isCLI)
 {
   ProgramInterface api("Infomap",
@@ -190,15 +196,15 @@ Config::Config(std::string flags, bool isCLI) : isCLI(isCLI)
   }
 
   if (flowModelArg == "undirected") {
-    flowModel = FlowModel::undirected;
+    setFlowModel(FlowModel::undirected);
   } else if (flowModelArg == "directed") {
-    flowModel = FlowModel::directed;
+    setFlowModel(FlowModel::directed);
   } else if (flowModelArg == "undirdir") {
-    flowModel = FlowModel::undirdir;
+    setFlowModel(FlowModel::undirdir);
   } else if (flowModelArg == "outdirdir") {
-    flowModel = FlowModel::outdirdir;
+    setFlowModel(FlowModel::outdirdir);
   } else if (flowModelArg == "rawdir") {
-    flowModel = FlowModel::rawdir;
+    setFlowModel(FlowModel::rawdir);
   }
 
   // Some checks
@@ -231,13 +237,13 @@ void Config::adaptDefaults()
   }
 
   if (undirdir) {
-    flowModel = FlowModel::undirdir;
+    setFlowModel(FlowModel::undirdir);
   } else if (directed) {
-    flowModel = FlowModel::directed;
+    setFlowModel(FlowModel::directed);
   } else if (outdirdir) {
-    flowModel = FlowModel::outdirdir;
+    setFlowModel(FlowModel::outdirdir);
   } else if (rawdir) {
-    flowModel = FlowModel::rawdir;
+    setFlowModel(FlowModel::rawdir);
   }
 
   auto outputs = io::split(outputFormats, ',');
